/************************************************************************
 *                                                                      *
 * Project: KAF - Khelben Arcade Frontend                               *
 *                                                                      *
 * Author: Marco Mastroddi <marco.mastroddi@libero.it>                  *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 ************************************************************************/

/*
 *    File    : buildgameslist.h
 */




#ifndef KAF_BUILDGAMESLIST_H
#define KAF_BUILDGAMESLIST_H

#include "game.h"

#include <qthread.h>
#include <qstring.h>
#include <qobject.h>
#include <qxml.h>

class Options;
class QTextStream;
class QFile;


class XmlGameParser : public QObject, public QXmlDefaultHandler
{
  Q_OBJECT
public:
  XmlGameParser();
  virtual ~XmlGameParser();

  bool startDocument();
  bool startElement( const QString&, const QString&, const QString& ,
                     const QXmlAttributes& );
  bool endElement( const QString&, const QString&, const QString& );

  bool characters( const QString& ch );

protected:
  enum State { XS_Unknown = 0, XS_Description, XS_Manufacturer, XS_Year };

signals:
  void gameFound( Game );

private:
  Game m_game;
  State m_state;
  QString m_buffer;
};


class BuildGamesList : public QObject, public QThread
{
  Q_OBJECT

public:
  enum State { Init = 0, Dumping, Dumped, Parsing, Parsed, Finished, Error };

  BuildGamesList( Options*, QObject* parent = 0 );
  virtual ~BuildGamesList();

  QString lastError() const { return m_error; }

  int state() const { return m_state; }

  void run();

signals:
  void stateChanged( int );

protected:
  void setError( const QString& );
  void setState( int );

  bool dumpToXmlFile();
  bool checkXmlFile( QFile* );


protected slots:
  void saveGame( Game );

private:
  Options* mp_options;
  int m_state;
  QString m_error;
  QTextStream* mp_stream;

};

#endif /* KAF_BUILDGAMESLIST_H */

