/************************************************************************
 *                                                                      *
 * Project: KAF - Khelben Arcade Frontend                               *
 *                                                                      *
 * Author: Marco Mastroddi <marco.mastroddi@libero.it>                  *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 ************************************************************************/

/*
 *    File    : gameitem.cpp
 */




#include "gameitem.h"
#include "game.h"
#include "icotoxpm.h"
#include "options.h"
#include "pixmaps.h"

#include <qobject.h>


#define COL_DESCRIPTION       0
#define COL_MANUFACTURER      1
#define COL_YEAR              2
#define COL_NAME              3
#define COL_CLONEOF           4
#define COL_TYPE              5
#define COL_CATEGORY          6

#define NUM_ATTRIBUTES        7



GameItem::GameItem( QListView* parent, Game* pGame )
  : QListViewItem( parent ), mp_game( pGame )
{
  refreshFields();
}

GameItem::~GameItem()
{
  //qDebug( "Deleting GameItem '%s'.", text( COL_NAME ).latin1() );
  delete mp_game;
}

int GameItem::countAttributes()
{
  return NUM_ATTRIBUTES;
}

int GameItem::attributeWidth( int index )
{
  if( index < 0 )
  {
    qWarning( "Invalid index found in GameItem::getAttributeName." );
    index = countAttributes();
  }

  switch( index )
  {
  case COL_DESCRIPTION:     return 250;
  case COL_MANUFACTURER:    return 100;
  case COL_YEAR:            return 60;
  case COL_NAME:            return 70;
  case COL_CLONEOF:         return 70;
  case COL_TYPE:            return 60;
  case COL_CATEGORY:        return 70;
  default:
    return -1;
  };
}

QString GameItem::attributeName( int index )
{
  if( index < 0 )
  {
    qWarning( "Invalid index found in GameItem::getAttributeName." );
    index = countAttributes();
  }

  switch( index )
  {
  case COL_DESCRIPTION:     return QObject::trUtf8( "Description" );
  case COL_MANUFACTURER:    return QObject::trUtf8( "Manufacturer" );
  case COL_YEAR:            return QObject::trUtf8( "Year" );
  case COL_NAME:            return QObject::trUtf8( "Name" );
  case COL_CLONEOF:         return QObject::trUtf8( "Cloneof" );
  case COL_TYPE:            return QObject::trUtf8( "Type" );
  case COL_CATEGORY:        return QObject::trUtf8( "Category" );
  default:
    return QObject::trUtf8( "Unknown" );
  };
}

void GameItem::refreshFields()
{
  if( !mp_game )
  {
    qWarning( "Invalid Game pointer in GameItem::init()." );
    setText( COL_DESCRIPTION, "Invalid game" );
  }

  setText( COL_DESCRIPTION, mp_game->description() );
  setText( COL_MANUFACTURER, mp_game->manufacturer() );
  setText( COL_YEAR, mp_game->year() );
  setText( COL_NAME, mp_game->name() );
  setText( COL_CLONEOF, mp_game->cloneof() );
  setText( COL_TYPE, mp_game->hasFlag( Game::Vector ) ? QObject::trUtf8( "Vector" ) : QObject::trUtf8( "Raster" )  );
  setText( COL_CATEGORY, mp_game->category() );
  setIcon();
}

void GameItem::setIcon()
{
  if( mp_game->hasFlag( Game::Available ) )
  {
    if( mp_game->hasFlag( Game::Favorite ) )
      setPixmap( 0, Pixmaps::getFavoriteIcon() );
    else if( mp_game->hasFlag( Game::Clone ) )
      setPixmap( 0, Pixmaps::getCloneIcon() );
    else
    {
      /*
      QString sFileIco = "/home/marco/tmp/mame-icons/";
      sFileIco += mp_game->name();
      sFileIco += ".xpm";
      QPixmap pix( sFileIco );
      if( !pix.isNull() )
        setPixmap( 0, pix );
      else
      */
        setPixmap( 0, Pixmaps::getRomIcon() );
    }
  }
  else
    setPixmap( 0, Pixmaps::getNoRomIcon() );
}

#if 0
void GameItem::loadIcon( Options* pOptions )
{
  pOptions->setGroup( G_MAME );

  QString sFileIco = "";
  sFileIco.sprintf( "%s/%s.ico",
    pOptions->getString( O_MAME_ICONS_DIR, "icons" ).latin1(),
    mp_game->name().latin1() );
  QPixmap pix = IcoToXpm::convert( sFileIco );
  if( pix.isNull() )
  {
    setIcon();
    return;
  }

  setPixmap( 0, pix );
}
#endif
