/************************************************************************
 *                                                                      *
 * Project: KAF - Khelben Arcade Frontend                               *
 *                                                                      *
 * Author: Marco Mastroddi <marco.mastroddi@libero.it>                  *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 ************************************************************************/

/*
 *    File    : gameslist.h
 */




#ifndef KAF_GAMESLIST_H
#define KAF_GAMESLIST_H

#include <qstringlist.h>


class Game;
class GameItem;
class QListView;
class QListViewItem;


class GamesList
{
public:
  GamesList( QListView* );
  virtual ~GamesList();

  void addGame( Game* );

  void setFilter( unsigned int filter ) { m_filter = filter; }
  unsigned int filter() const { return m_filter; }

  int total() const { return m_total; }
  int availables() const { return m_availables; }
  int favorites() const { return m_favorites.count(); }

  void setAvailable( GameItem* );
  void unsetAvailable( GameItem* );
  void setFavorite( GameItem* );
  void unsetFavorite( GameItem* );

  void refreshFiltered();
  void refreshAllGames();

  void clear();

  bool isFavorite( const QString& );
  bool saveFavorites( const QString& );
  bool loadFavorites( const QString& );

  Game* game( const QString& );

  void addCategory( const QString& );
  const QStringList& categories() const { return m_categories; }
  const QString& categoryFilter() const { return m_categoryFilter; }
  void setCategoryFilter( const QString& cat ) { m_categoryFilter = cat; }
  void sortCategories() { m_categories.sort(); }

protected:
  void checkFlags( Game* );
  void setTitle();
  bool canBeShowedInList( QListViewItem* );
  bool hasCategory( const QString& );

private:
  QListView* mp_list;
  QStringList m_favorites;
  QStringList m_categories;
  int m_availables;
  int m_total;
  unsigned int m_filter;
  QString m_categoryFilter;

};

#endif /* KAF_GAMESLIST_H */
