/************************************************************************
 *                                                                      *
 * Project: KAF - Khelben Arcade Frontend                               *
 *                                                                      *
 * Author: Marco Mastroddi <marco.mastroddi@libero.it>                  *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 ************************************************************************/

/*
 *    File    : icotoxpm.cpp
 */




#include "icotoxpm.h"

#include <qstring.h>
#include <stdio.h>
#include <stdlib.h>

#include <qfile.h>
#include <qtextstream.h>


#ifndef MAX_PATH_LEN
#define MAX_PATH_LEN 1024
#endif


/* The maximum number of icons we'll try to handle in any one file */
#define MAXICONS 32

/* Header of a single icon image in an icon file */
typedef struct IconDirectoryEntry
{
  unsigned char  bWidth;
  unsigned char  bHeight;
  unsigned char  bColorCount;
  unsigned char  bReserved;
  unsigned short  wPlanes;
  unsigned short  wBitCount;
  unsigned long  dwBytesInRes;
  unsigned long  dwImageOffset;
} ICONDIRENTRY;

/* Header of an icon file */
typedef struct ICONDIR
{
  unsigned short          idReserved;
  unsigned short          idType;
  unsigned short          idCount;
/*    ICONDIRENTRY  idEntries[1];*/
  ICONDIRENTRY  idEntries[MAXICONS];
} ICONHEADER;

/* Bitmap header - this is on the images themselves */
typedef struct tagBITMAPINFOHEADER{
  unsigned long      biSize;
  long               biWidth;
  long               biHeight;
  unsigned short     biPlanes;
  unsigned short     biBitCount;
  unsigned long      biCompression;
  unsigned long      biSizeImage;
  long               biXPelsPerMeter;
  long               biYPelsPerMeter;
  unsigned long      biClrUsed;
  unsigned long      biClrImportant;
} BITMAPINFOHEADER;

/* Magic number for an icon file */
/* This is the the idReserved and idType fields of the header */
static char ico_magic_number[4] = {0, 0, 1, 0};

static char *program_name;
static char input_file [MAX_PATH_LEN];

/* For keeping track of which icon image we're on */
static int whichimage = 0;


/*
 * Conversion
 */

#define XPM_HEAD "/* XPM */\n\
static const char *pixmap[] = {\n\
/* width height ncols cpp */\n\
\"%d %d %d 2\",\n  /* Colors */\n"

/* The smallest number of double words containing c bytes */
#define DWORD_ALIGN_BYTES(b) (((b) / 4) + (((b) % 4 > 0) ? 1 : 0))

/* The smallest number of double words containing c bits */
#define DWORD_ALIGN_BITS(b) (((b) / 32) + (((b) % 32 > 0) ? 1 : 0))

/* The different possible number of colors */
#define LOW_COLOR 16
#define HIGH_COLOR 0
/*#define NCOLORS (color_level == LOW_COLOR ? 16 : 256)*/
#define NCOLORS ncolors

/* The color index of a pixel in the icon colormap */
#define PIXEL_INDEX(x,y) \
     (color_level == LOW_COLOR \
      ? ((x) % 2 == 0 \
	 ? (image [bytes_per_line * (height - 1 - (y)) / 2 + (x) / 2] & 0xF0) >> 4 \
	 : image [bytes_per_line * (height - 1 - (y)) / 2 + (x) / 2] & 0x0F) \
      : image [bytes_per_line * (height - 1 - (y)) + (x)])
     
/* The color index of a pixel in the (reduced) pixmap colormap */
#define PIXEL_REDUCED_INDEX(x,y) \
     (reduced_colormap_index [PIXEL_INDEX((x),(y))])
  
/* The RGB value of a colormap entry */
#define INDEX_VALUE(index,c) (colormap[(c) + (index) * 4])
#define INDEX_R_VALUE(index) (INDEX_VALUE((index), 2))
#define INDEX_G_VALUE(index) (INDEX_VALUE((index), 1))
#define INDEX_B_VALUE(index) (INDEX_VALUE((index), 0))
  
/* The transparency of a pixel */
#define MASK_BYTE(x,y) (bytes_per_mask_line * (height - 1 - (y)) + (x) / 8)
#define MASK_BIT(x,y) (7 - (x) % 8)
#define IS_TRANSPARENT(x,y) \
     (mask[MASK_BYTE((x),(y))] & (1 << MASK_BIT((x),(y))))
  
  
static FILE *ico_stream;

/* Values read from icon file */
static ICONHEADER iconheader;
static BITMAPINFOHEADER bitmapinfoheader;
/*static unsigned char magic_number [sizeof(ico_magic_number)];*/
static unsigned char *magic_number;
//static unsigned char cwidth;
//static unsigned char cheight;
static unsigned char color_level;
//static unsigned char unknown_part [53];
static unsigned char colormap [256 * 4];
static unsigned char image [256 * 256]; 
static unsigned char mask [256 * 32];

/* Different variables computed from the read ones */
static int ncolors;
static int width;
static int height;
static unsigned char bytes_per_line;
static unsigned char bytes_per_mask_line;
static int image_length;
static int mask_length;

/* Variables for the reduced colormap */
static int color_used[256];
static int nb_color_used;
static unsigned char reduced_colormap_index [256];

/* Does the icon have transparent pixels ? */
static int have_transparent_pixels;


QPixmap IcoToXpm::convert( const QString& file_name )
{
  /*
   * Read the icon file
   */

  int x, y, i;

  if( (ico_stream = fopen( file_name.latin1(), "r") ) == NULL )
  {
    return QPixmap();
  }

  qDebug( "Loading icon '%s'...", file_name.latin1() );

  /* Check the magic number & header */
  /*fread (magic_number, 1, sizeof(ico_magic_number), ico_stream);*/
  fread( &iconheader, 1, 6/*sizeof(unsigned short)*3*/, ico_stream );
  magic_number = (unsigned char *)&iconheader;
  for (i = 0 ; (unsigned int)i < sizeof(ico_magic_number) ; i ++)
  {
    if (magic_number [i] != ico_magic_number [i])
    {
      qDebug( "Loading icon '%s'... failed!", file_name.latin1() );
      return QPixmap();
    }
  }

  /* Read in the rest of the icon directory entries */
  fread( &iconheader.idEntries[0], iconheader.idCount,
    /*sizeof(ICONDIRENTRY)*/16, ico_stream );

  /* Cycle through each icon image */
  for( whichimage = 0; whichimage < iconheader.idCount; whichimage++ ) 
  {
    /* Read the icon size */
    width = iconheader.idEntries[whichimage].bWidth;
    height = iconheader.idEntries[whichimage].bWidth;

    /* Determine the number of bytes defining a line of the icon. */
    bytes_per_line = 4 * DWORD_ALIGN_BYTES (width);
    bytes_per_mask_line = 4 * DWORD_ALIGN_BITS (width);

    /* Let's surf on over to the bitmap image & read the BMIH. */
    fseek (ico_stream, iconheader.idEntries[whichimage].dwImageOffset,SEEK_SET);
    fread (&bitmapinfoheader, 1, sizeof(bitmapinfoheader), ico_stream);

    switch( bitmapinfoheader.biPlanes * bitmapinfoheader.biBitCount)
    {
      case 4: /* 2^4 = 14 */ color_level = LOW_COLOR; break;
      case 8: /* 2^8 = 256 */ color_level = HIGH_COLOR; break;
      default:
        qDebug( "%s: unsupported number of colors in image! Skipping. (%d planes, %d bpp)\n",
          input_file,
          (int)bitmapinfoheader.biPlanes,
          (int)bitmapinfoheader.biBitCount );
        continue;
    }

    /* Read the colormap */
    if (bitmapinfoheader.biClrImportant != 0)
      ncolors = bitmapinfoheader.biClrImportant;
    else if (bitmapinfoheader.biClrUsed != 0)
      ncolors = bitmapinfoheader.biClrUsed;
    else
      ncolors = 1 << (bitmapinfoheader.biPlanes*bitmapinfoheader.biBitCount);

    fseek (ico_stream, iconheader.idEntries[whichimage].dwImageOffset
      + bitmapinfoheader.biSize, SEEK_SET);
    fread (colormap, 1,ncolors * 4, ico_stream);

    /* Read the image */
    if (color_level == LOW_COLOR)
        image_length = bytes_per_line * height / 2;
    else
        image_length = bytes_per_line * height;
    fread (image, 1, image_length, ico_stream);

    /* Read the mask */
    mask_length = bytes_per_mask_line * height;
    fread (mask, 1, mask_length, ico_stream);

    /* Read error / prematured EOF  occured ? */
    if (feof (ico_stream))
      qDebug( "%s: %s: Premature end-of-file.\n",
      program_name, input_file);
    if (ferror (ico_stream))
      qDebug( "%s: %s: Read error.\n",
      program_name, input_file);
    if (feof (ico_stream) || ferror (ico_stream))
      {
        fclose (ico_stream);
        return QPixmap();
      }

    /* Reduce the number of colors */
    for (i = 0 ; i < ncolors ; i++)
      color_used [i] = 0;

    for (y = 0 ; y <height ; y++)
      for (x = 0 ; x < width ; x++)
        color_used [PIXEL_INDEX(x,y)] = 1;

    nb_color_used = 0;
    for (i = 0 ; i < ncolors ; i++)
      if (color_used [i])
        reduced_colormap_index[i] = nb_color_used ++;

    /* Check for at least one transparent pixel */
    have_transparent_pixels = 0;
    for (y=0; y<height && have_transparent_pixels == 0 ; y++)
    {
      for (x=0; x<width; x++)
      {
        if (IS_TRANSPARENT(x,y))
        {
          have_transparent_pixels = 1;
          break;
        }
      }
    }

    /*
     * Write the pixmap file
     */

    QString sXPM = "";
    QString sTmp = "";


    sTmp.sprintf( XPM_HEAD, width, height, nb_color_used + have_transparent_pixels );

    sXPM += sTmp;

    /* Write the colors */
    x = 0;

    for (i=0; i<NCOLORS; i++)
    {
      if (color_used [i])
      {
        sTmp = "";
        sTmp.sprintf( "  \"%.2X c #%.2X%.2X%.2X\",\n", x++,
          INDEX_R_VALUE(i), INDEX_G_VALUE(i), INDEX_B_VALUE(i));
        sXPM += sTmp;
      }
    }

    if(have_transparent_pixels)
      sXPM += "  \".. s None c None\",\n";

    /* Write the image */
    for (y=0; y<height; y++)
    {
      sXPM += "  \"";
      for (x=0; x<width; x++)
      {
        if (IS_TRANSPARENT(x,y))
          sXPM += "..";
        else
        {
          sTmp = "";
          sTmp.sprintf( "%.2X", PIXEL_REDUCED_INDEX(x,y) );
          sXPM += sTmp;
        }
      }
      sXPM += "\",\n";
    }

    sXPM += "};\n";

    QFile file( "/tmp/kafico.tmp" );
    if( file.open( IO_WriteOnly ) )
    {
      QTextStream ts( &file );
      ts << sXPM << endl << flush;
      file.close();
      QPixmap pix( file.name() );
      file.remove();
      return pix;
    }
  }

  return QPixmap();
}
