/************************************************************************
 *                                                                      *
 * Project: KAF - Khelben Arcade Frontend                               *
 *                                                                      *
 * Author: Marco Mastroddi <marco.mastroddi@libero.it>                  *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 ************************************************************************/

/*
 *    File    : main.cpp
 */




#include "wndmainwindow.h"

#include <qapplication.h>
#include <qdesktopwidget.h>
#include <qrect.h>

#include <stdio.h>
#include <stdlib.h>


static bool EnableDebugMessage = false;


void KAFMessageHandler( QtMsgType type, const char *msg )
{
  switch( type )
  {
  case QtDebugMsg:
    if( EnableDebugMessage )
      fprintf( stderr, "Debug: %s\n", msg );
    break;
  case QtWarningMsg:
    fprintf( stderr, "Warning: %s\n", msg );
    break;
  case QtFatalMsg:
    fprintf( stderr, "Error: %s\n", msg );
    abort();                    // deliberately core dump
  }
}

static void CheckEnvironment()
{
#if defined( Q_WS_X11 )
  char* pEnv = getenv( "KAF_DEBUG" );
  if( pEnv != NULL )
  {
    QString sEnv( pEnv );
    if( sEnv == "true" || sEnv == "1" || sEnv == "yes" )
      EnableDebugMessage = true;
  }
#endif
}

int main( int argc, char** argv )
{
  CheckEnvironment();

  qInstallMsgHandler( KAFMessageHandler );

  QApplication kaf( argc, argv );

  WndMainWindow *pMainWindow = new WndMainWindow();

  kaf.setMainWidget( pMainWindow );

  pMainWindow->resize( 640, 480 );

#if QT_VERSION >= 0x030100
  QRect rect = (kaf.desktop())->availableGeometry();
#else
  QRect rect = (kaf.desktop())->screenGeometry();
#endif
  int w = rect.width() - pMainWindow->width();
  int h = rect.height() - pMainWindow->height();
  pMainWindow->move( w/2, h/2 );

  pMainWindow->show();

  int iRetValue = kaf.exec();

  delete pMainWindow;

  return iRetValue;
}

