/************************************************************************
 *                                                                      *
 * Project: KAF - Khelben Arcade Frontend                               *
 *                                                                      *
 * Author: Marco Mastroddi <marco.mastroddi@libero.it>                  *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 ************************************************************************/

/*
 *    File    : mameprocess.h
 */




#ifndef KAF_MAMEPROCESS_H
#define KAF_MAMEPROCESS_H

#include <qprocess.h>

class Game;
class Options;
class QString;


class MameProcess : public QProcess
{
  Q_OBJECT

public:
  MameProcess( Options* );
  ~MameProcess();

  bool errorFound() { return m_errorFound; }

  Game* game() const { return mp_game; }

  void setGameArguments( Game* );

  int checkVersion(); /* -1 Mame not found
                          0 version not changed
                          1 build or render changed
                          2 version changed */

  bool currentMameVersionIsGreaterThan( const QString& );

  void checkSupportedGames();

  void waitTheEnd();

public slots:
  void slotStartGame();

protected slots:
  void slotCheckLog();

signals:
  void logAvailable( const QString& );
  void processStarted();

protected:
  void setPathArguments();
  void setX11Arguments();
  void setXGLArguments();
  void setSDLArguments();
  void setDisplayArguments();
  void setSoundArguments();
  void setNetworkArguments();
  void setInputArguments();
  void setVectorArguments();
  void setMiscArguments();

private:
  Game* mp_game;
  Options* mp_options;
  bool m_errorFound;
  QString m_render;

};

#endif /* KAF_MAMEPROCESS_H */
