/************************************************************************
 *                                                                      *
 * Project: KAF - Khelben Arcade Frontend                               *
 *                                                                      *
 * Author: Marco Mastroddi <marco.mastroddi@libero.it>                  *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 ************************************************************************/

/*
 *    File    : wndmainwindow.h
 */




#ifndef KAF_WNDMAINWINDOW_H
#define KAF_WNDMAINWINDOW_H


#include "guimainwindow.h"

class Options;
class GamesList;
class QAction;
class QListViewItem;
class QString;
class QPopupMenu;
class Game;
class QProgressDialog;
class QTextStream;
class WndInfo;



class WndMainWindow : public GuiMainWindow
{
  Q_OBJECT
public:
  enum PreviewMode { SnapshotsPreview = 0, FlyersPreview };

  WndMainWindow( QWidget* parent = 0 );
  ~WndMainWindow();

protected slots:
  void slotLoadGamesList();
  void slotBuildGamesListActivated();
  void slotRefreshGamesList();
  void slotSearchForAvailablesActivated();
  void slotSearchForAvailables();
  void slotShowAbout();
  void slotGameSelected( QListViewItem* );
  void slotStartGameSelected( QListViewItem* );
  void slotMamePathsActivated();
  void slotGameOptionsActivated();
  void slotStartGame();
  void slotChangeFilter( int );
  void slotRightButtonPressed( QListViewItem*, const QPoint&, int );
  void slotChangeFavoriteOptionForGame();
  void slotProgressCancelled();
  void slotCheckMameVersion();
  void slotReadMameInfoActivated();
  void slotReadMameInfo();
  void slotShowInfoWidget();
  void slotDeleteInfoWidget();
  void slotShowMameInfo();
  void slotCategorySelected( const QString& );
  void slotChangePreviewMode();
  void slotShowPreferences();

protected:
  void initGamesListView();

  void setMessage( const QString& );
  void setStandardMessage();
  void setStandardMessage( Game* );
  void updateLabelVersion();

  void activateProgressBar( int );
  void advanceProgressBar( int );

  void setGamesListTitle();

  bool loadGamesList();
  bool saveGamesList();
  bool loadGamesListHeader( QTextStream* );
  void saveGamesListHeader( QTextStream* );

  void changeFilter( int id );

  void updateCategories( bool );

  bool romsDirectoryExists();
  bool mameBinaryExists();
  bool gamesListIsEmpty();

  void showPreview( Game* );
  void updatePreview();

  void showItemSelected();

private:
  Options* mp_options;
  GamesList* mp_games;

  QAction* mp_actStartGame;
  QAction* mp_actCheckMameVersion;
  QAction* mp_actReadMameInfo;
  QAction* mp_actBuildList;
  QAction* mp_actSearchAvailables;
  QAction* mp_actMamePaths;
  QAction* mp_actGameOptions;

  QPopupMenu* mp_mameMenu;
  QPopupMenu* mp_filterMenu;

  QProgressDialog* mp_progress;
  bool m_progressCancelled;

  WndInfo* mp_info;

  PreviewMode m_previewMode;

};

#endif /* KAF_WNDMAINWINDOW_H */
