/************************************************************************
 *                                                                      *
 * Project: KAF - Khelben Arcade Frontend                               *
 *                                                                      *
 * Author: Marco Mastroddi <marco.mastroddi@libero.it>                  *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 ************************************************************************/

/*
 *    File    : wndmamepaths.cpp
 */




#include "wndmamepaths.h"
#include "options.h"
#include "pixmaps.h"

#include <qpushbutton.h>
#include <qlineedit.h>
#include <qfiledialog.h>
#if QT_VERSION < 0x030100
#include <qregexp.h>
#endif


WndMamePaths::WndMamePaths( Options* pOptions, QWidget* parent )
  : GuiMamePaths( parent, "MamePathsDialog", true )
{
  setIcon( Pixmaps::getMameIcon() );

  m_mamePathChanged = false;

  mp_options = pOptions;

  connect( bOk, SIGNAL( clicked() ), this, SLOT( slotOk() ) );
  connect( bCancel, SIGNAL( clicked() ), this, SLOT( slotCancel() ) );

  connect( bMameExe, SIGNAL( clicked() ), this, SLOT( slotSelectMameExe() ) );
  connect( bRomsDir, SIGNAL( clicked() ), this, SLOT( slotSelectRomsDir() ) );
  connect( bIconsDir, SIGNAL( clicked() ), this, SLOT( slotSelectIconsDir() ) );
  connect( bSnapshotsDir, SIGNAL( clicked() ), this, SLOT( slotSelectSnapshotsDir() ) );
  connect( bSamplesDir, SIGNAL( clicked() ), this, SLOT( slotSelectSamplesDir() ) );

  connect( bArtworkDir, SIGNAL( clicked() ), this, SLOT( slotSelectArtworkDir() ) );
  connect( bFlyersDir, SIGNAL( clicked() ), this, SLOT( slotSelectFlyersDir() ) );
  connect( bIniDir, SIGNAL( clicked() ), this, SLOT( slotSelectIniDir() ) );
  connect( bCtrlrDir, SIGNAL( clicked() ), this, SLOT( slotSelectCtrlrDir() ) );
  connect( bHiscoreFile, SIGNAL( clicked() ), this, SLOT( slotSelectHiscoreFile() ) );
  connect( bCheatFile, SIGNAL( clicked() ), this, SLOT( slotSelectCheatFile() ) );
  connect( bHistoryFile, SIGNAL( clicked() ), this, SLOT( slotSelectHistoryFile() ) );
  connect( bInfoFile, SIGNAL( clicked() ), this, SLOT( slotSelectInfoFile() ) );

  updateMenus();
}

WndMamePaths::~WndMamePaths()
{

}

void WndMamePaths::removeLastSlashFromPath( QString* pPath )
{
#if QT_VERSION >= 0x030100
  pPath->replace( "\\", "/" );
#else
  pPath->replace( QRegExp( "\\" ), "/" );
#endif
  int iLastChar = pPath->length() - 1;
  if( pPath->at( iLastChar ) == '/' || pPath->at( iLastChar ) == '\\' )
    pPath->remove( iLastChar, 1 );
}

void WndMamePaths::updateMenus()
{
  mp_options->setGroup( G_MAME );
  leMameExe->setText( mp_options->getString( O_MAME_EXE, "/usr/local/bin/xmame.x11" ) );
  leRomsDir->setText( mp_options->getString( O_MAME_ROMS_DIR, "/usr/local/games/mame/roms" ) );
  leIconsDir->setText( mp_options->getString( O_MAME_ICONS_DIR, "/usr/local/games/mame/icons" ) );
  leSnapshotsDir->setText( mp_options->getString( O_MAME_SNAPS_DIR, "/usr/local/games/mame/snap" ) );
  leSamplesDir->setText( mp_options->getString( O_MAME_SAMPLES_DIR, "/usr/local/games/mame/samples" ) );
  leIniDir->setText( mp_options->getString( O_MAME_INI_DIR, "/usr/local/games/mame/ini" ) );
  leArtworkDir->setText( mp_options->getString( O_MAME_ARTWORK_DIR, "/usr/local/games/mame/artwork" ) );
  leFlyersDir->setText( mp_options->getString( O_MAME_FLYERS_DIR, "/usr/local/games/mame/flyers" ) );
  leCtrlrDir->setText( mp_options->getString( O_MAME_CTRLR_DIR, "/usr/local/games/mame/ctrlr" ) );
  leHiscoreFile->setText( mp_options->getString( O_MAME_HISCORE_FILE, "/usr/local/games/mame/hiscore.dat" ) );
  leCheatFile->setText( mp_options->getString( O_MAME_CHEAT_FILE, "/usr/local/games/mame/cheat.dat" ) );
  leHistoryFile->setText( mp_options->getString( O_MAME_HISTORY_FILE, "/usr/local/games/mame/history.dat" ) );
  leInfoFile->setText( mp_options->getString( O_MAME_INFO_FILE, "/usr/local/games/mame/mameinfo.dat" ) );

  m_mamePathChanged = false;
}

void WndMamePaths::setDirInOptions( Options *pOptions, const QString& key, const QString& value )
{
  QString sTmp = value.simplifyWhiteSpace();
  removeLastSlashFromPath( &sTmp );
  pOptions->setString( key, sTmp );
}

void WndMamePaths::slotOk()
{
  mp_options->setGroup( G_MAME );
  QString sTmp = mp_options->getString( O_MAME_EXE, "/usr/local/bin/xmame" );
  m_mamePathChanged = ( sTmp != leMameExe->text().simplifyWhiteSpace() );
  mp_options->setString( O_MAME_EXE, leMameExe->text().simplifyWhiteSpace() );
  setDirInOptions( mp_options, O_MAME_ROMS_DIR, leRomsDir->text() );
  setDirInOptions( mp_options, O_MAME_ICONS_DIR, leIconsDir->text() );
  setDirInOptions( mp_options, O_MAME_SNAPS_DIR, leSnapshotsDir->text() );
  setDirInOptions( mp_options, O_MAME_SAMPLES_DIR, leSamplesDir->text() );
  setDirInOptions( mp_options, O_MAME_INI_DIR, leIniDir->text() );
  setDirInOptions( mp_options, O_MAME_ARTWORK_DIR, leArtworkDir->text() );
  setDirInOptions( mp_options, O_MAME_FLYERS_DIR, leFlyersDir->text() );
  setDirInOptions( mp_options, O_MAME_CTRLR_DIR, leCtrlrDir->text() );
  mp_options->setString( O_MAME_HISCORE_FILE, leHiscoreFile->text().simplifyWhiteSpace() );
  mp_options->setString( O_MAME_CHEAT_FILE, leCheatFile->text().simplifyWhiteSpace() );
  mp_options->setString( O_MAME_HISTORY_FILE, leHistoryFile->text().simplifyWhiteSpace() );
  mp_options->setString( O_MAME_INFO_FILE, leInfoFile->text().simplifyWhiteSpace() );

  done( 1 );
}

void WndMamePaths::slotCancel()
{
  done( 0 );
}

bool WndMamePaths::selectDir( QLineEdit* ple, const QString& txt )
{
  QString sDir = QFileDialog::getExistingDirectory( ple->text(), this,
                   "SelectDirDialog", txt, true, false );

  if( sDir.isNull() || sDir.isEmpty() )
    return false;

  ple->setText( sDir );

  return true;
}

bool WndMamePaths::selectFile( QLineEdit* ple, const QString& txt )
{
  QString sFile = QFileDialog::getOpenFileName( ple->text(), QString::null, this,
                    "SelectFileDialog", txt, 0, false );

  if( sFile.isNull() || sFile.isEmpty() )
    return false;

  ple->setText( sFile );

  return true;
}

void WndMamePaths::slotSelectMameExe()
{
  selectFile( leMameExe, trUtf8( "KAF - Select MaME executable" ) );
}

void WndMamePaths::slotSelectRomsDir()
{
  selectDir( leRomsDir, trUtf8( "KAF - Select MaME roms directory" ) );
}

void WndMamePaths::slotSelectIconsDir()
{
  selectDir( leIconsDir, trUtf8( "KAF - Select MaME icons directory" ) );
}

void WndMamePaths::slotSelectSnapshotsDir()
{
  selectDir( leSnapshotsDir, trUtf8( "KAF - Select MaME snapshots directory" ) );
}

void WndMamePaths::slotSelectSamplesDir()
{
  selectDir( leSamplesDir, trUtf8( "KAF - Select MaME samples directory" ) );
}

void WndMamePaths::slotSelectArtworkDir()
{
  selectDir( leArtworkDir, trUtf8( "KAF - Select MaME artwork directory" ) );
}

void WndMamePaths::slotSelectFlyersDir()
{
  selectDir( leFlyersDir, trUtf8( "KAF - Select MaME flyers directory" ) );
}

void WndMamePaths::slotSelectIniDir()
{
  selectDir( leIniDir, trUtf8( "KAF - Select MaME ini directory" ) );
}

void WndMamePaths::slotSelectCtrlrDir()
{
  selectDir( leCtrlrDir, trUtf8( "KAF - Select MaME ctrlr directory" ) );
}

void WndMamePaths::slotSelectHiscoreFile()
{
  selectFile( leHiscoreFile, trUtf8( "KAF - Select MaME hiscore file" ) );
}

void WndMamePaths::slotSelectCheatFile()
{
  selectFile( leCheatFile, trUtf8( "KAF - Select MaME cheat file" ) );
}

void WndMamePaths::slotSelectHistoryFile()
{
  selectFile( leHistoryFile, trUtf8( "KAF - Select MaME history file" ) );
}

void WndMamePaths::slotSelectInfoFile()
{
  selectFile( leInfoFile, trUtf8( "KAF - Select MaME info file" ) );
}



