/************************************************************************
 *                                                                      *
 * Project: KAF - Khelben Arcade Frontend                               *
 *                                                                      *
 * Author: Marco Mastroddi <marco.mastroddi@libero.it>                  *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 ************************************************************************/

/*
 *    File    : wndmameprocess.cpp
 */




#include "wndmameprocess.h"
#include "mameprocess.h"
#include "game.h"
#include "options.h"
#include "pixmaps.h"

#include <qpushbutton.h>
#include <qtextedit.h>
#include <qtimer.h>
#include <qstring.h>


WndMameProcess::WndMameProcess( Game* pGame, Options* pOptions, QWidget* parent )
  : GuiMameProcess( parent, "MameProcessWindow", true )
{
  setIcon( Pixmaps::getMameIcon() );

  mp_mame = new MameProcess( pOptions );
  mp_mame->setGameArguments( pGame );
  QString sTmp = "MaME - ";
  sTmp += pGame->description();
  setCaption( sTmp );

  connect( bCancel, SIGNAL( clicked() ), this, SLOT( slotCancel() ) );
  connect( mp_mame, SIGNAL( logAvailable( const QString& ) ), teLog, SLOT( append( const QString& ) ) );
  connect( mp_mame, SIGNAL( processExited() ), this, SLOT( slotProcessExited() ) );
  connect( mp_mame, SIGNAL( processStarted() ), this, SLOT( showMinimized() ) );

  QTimer::singleShot( 100, mp_mame, SLOT( slotStartGame() ) );
}

WndMameProcess::~WndMameProcess()
{
  delete mp_mame;
}

void WndMameProcess::slotCancel()
{
  if( mp_mame->isRunning() )
  {
    qDebug( "Closing game: '%s'.", mp_mame->game()->description().latin1() );
    mp_mame->kill();
  }

  done( 1 );
}

void WndMameProcess::slotProcessExited()
{
  if( mp_mame->errorFound() )
  {
    bCancel->setText( trUtf8("Close") );
    showNormal();
    return;
  }

  done( 0 );
}
